@@ -9,7 +9,7 @@ use 5.014;
 #use strict;
 #use warnings;
 
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 
 our %TRUE_VALUES = (
                     'true' => 1,
@@ -182,11 +182,21 @@ sub parse_desktop_file {
         } = ();
 #>>>
 
+    my @cats = keys %categories;
+
     # Skip entry when there are no categories and `keep_unknown_categories` is false
-    %categories or $self->{keep_unknown_categories} or return;
+    # When `keep_unknown_categories` is true, set `@cats` to `unknown_category_key`.
+    if (!@cats) {
+        if ($self->{keep_unknown_categories}) {
+            push @cats, $self->{unknown_category_key};
+        }
+        else {
+            return;
+        }
+    }
 
     # Store the categories
-    $info{Categories} = [keys %categories];
+    $info{Categories} = \@cats;
 
     # Remove `% ...` from the value of `Exec`
     index($info{Exec}, ' %') != -1 and $info{Exec} =~ s/ +%.*//s;
@@ -228,19 +238,11 @@ sub parse {
     my ($self, $hash_ref, @desktop_files) = @_;
 
     foreach my $desktop_file (@desktop_files) {
-        my %info = $self->parse_desktop_file($desktop_file);
-
-        # Skip when %info is empty
-        %info || next;
+        (my %info = $self->parse_desktop_file($desktop_file)) || next;
 
         # Push the entry into its belonging categories
-        if (exists($info{Categories}) and @{$info{Categories}}) {
-            foreach my $category (@{$info{Categories}}) {
-                push @{$hash_ref->{$category}}, {map { $_ => $info{$_} } @{$self->{keys_to_keep}}};
-            }
-        }
-        else {
-            push @{$hash_ref->{$self->{unknown_category_key}}}, {map { $_ => $info{$_} } @{$self->{keys_to_keep}}};
+        foreach my $category (@{$info{Categories}}) {
+            push @{$hash_ref->{$category}}, \%info;
         }
     }
 }
@@ -335,8 +337,7 @@ Sets the directories where to find the desktop files.
 
 =item keys_to_keep => [qw(Name Exec Icon Comment ...)]
 
-Any valid keys from the desktop files. This keys will be stored in the returned
-hash reference when calling C<$obj-E<gt>parse_desktop_files>.
+Any valid keys from the desktop files to keep in the results from C<parse_desktop_file>. The B<Categories> option is implicitly included.
 
 =item categories => [qw(Graphics Network AudioVideo ...)]
 
@@ -374,6 +375,8 @@ When the value of B<Terminal> is true, modify the B<Exec> value to something lik
 
     terminal -e 'command'
 
+This option will include the C<Terminal> key inside the B<keys_to_keep> array.
+
 =item terminalization_format => q{%s -e '%s'}
 
 Format used by C<sprintf()> to terminalize a command which requires to be executed
@@ -409,10 +412,13 @@ Example:
 =item substitutions => [{key => 'KeyName', re => qr/REGEX/i, value => 'Value'}, {...}]
 
 Substitute, by using a regex, in the values of the desktop files.
+
 The B<key> can be any valid key from the desktop files.
+
 The B<re> can be any valid regular expression. Anything matched by the regex, will be
 replaced the string stored in B<value>.
-For global matching/substitution, you need to set the B<global> key to a true value.
+
+For global matching/substitution, set the B<global> key to a true value.
 
 Example:
 
@@ -454,17 +460,20 @@ where C<%info> might look something like this:
         Categories => ["...", "...", "..."],
     );
 
+When B<keep_unknown_categories> is true and a given entry does not belong to any category,
+C<parse_desktop_file> will set B<Categories> to [C<unknown_category_key>].
+
 =item $obj->parse_desktop_files()
 
-It returns a HASH reference which categories names as keys, and ARRAY references
-as values which contains HASH references with the keys specified in the B<keys_to_keep>
+It returns a HASH reference categorized on category names, with ARRAY references
+as values, each ARRAY containing a HASH reference with the keys specified in the B<keys_to_keep>
 option, and values from the desktop files.
 
 The returned HASH reference might look something like this:
 
         {
-          utility => [ {Exec => "...", Name => "..."}, {Exec => "...", Name => "..."} ],
-          network => [ {Exec => "...", Name => "..."}, {Exec => "...", Name => "..."} ],
+          Utility => [ {Exec => "...", Name => "..."}, {Exec => "...", Name => "..."} ],
+          Network => [ {Exec => "...", Name => "..."}, {Exec => "...", Name => "..."} ],
         }
 
 This function is equivalent with:
